package com.fjhx.victoriatourist.service.jd.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.utils.Assert;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.item.util.excel.util.ExcelUtil;
import com.fjhx.victoriatourist.entity.jd.dto.JdRefundNotQualityCheckSelectDto;
import com.fjhx.victoriatourist.entity.jd.po.JdRefundNotQualityCheck;
import com.fjhx.victoriatourist.entity.jd.po.JdRefundQualityCheck;
import com.fjhx.victoriatourist.entity.jd.vo.JdRefundNotQualityCheckVo;
import com.fjhx.victoriatourist.mapper.jd.JdRefundNotQualityCheckMapper;
import com.fjhx.victoriatourist.service.jd.JdRefundNotQualityCheckService;
import com.fjhx.victoriatourist.service.jd.JdRefundQualityCheckService;
import com.fjhx.victoriatourist.utils.CodeEnum;
import com.fjhx.wms.entity.stock.po.Stock;
import com.fjhx.wms.service.stock.StockService;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.fjhx.victoriatourist.service.jd.impl.JdRefundServiceImpl.WAREHOUSE_ID;
import static com.fjhx.victoriatourist.service.jd.impl.JdRefundServiceImpl.getUpdateStockSql;


/**
 * <p>
 * 京东售后退货待质检 服务实现类
 * </p>
 *
 * @author
 * @since 2024-09-28
 */
@Service
public class JdRefundNotQualityCheckServiceImpl extends ServiceImpl<JdRefundNotQualityCheckMapper, JdRefundNotQualityCheck> implements JdRefundNotQualityCheckService {

    @Autowired
    private JdRefundQualityCheckService qualityCheckService;

    @Autowired
    private StockService stockService;

    @Autowired
    private ProductInfoService productInfoService;

    @Override
    public Page<JdRefundNotQualityCheckVo> getPage(JdRefundNotQualityCheckSelectDto dto) {
        IWrapper<JdRefundNotQualityCheck> wrapper = getWrapper();
        wrapper.orderByDesc("jrnqc", JdRefundNotQualityCheck::getId);
        wrapper.gt("jrnqc", JdRefundNotQualityCheck::getQuantity, 0);
        Page<JdRefundNotQualityCheckVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @DSTransactional
    @Override
    public synchronized void submitQualityCheck(List<JdRefundQualityCheck> dto) {

        Map<Long, JdRefundQualityCheck> qualityCheckMap = dto.stream()
                .peek(item -> item.setId(null))
                .peek(item -> item.setQualifiedCount(ObjectUtil.defaultIfNull(item.getQualifiedCount(), 0)))
                .peek(item -> item.setDefectiveCount(ObjectUtil.defaultIfNull(item.getDefectiveCount(), 0)))
                .peek(item -> item.setScrappedCount(ObjectUtil.defaultIfNull(item.getScrappedCount(), 0)))
                .filter(item -> item.getQualifiedCount() + item.getDefectiveCount() + item.getScrappedCount() > 0)
                .collect(Collectors.toMap(JdRefundQualityCheck::getProductId, Function.identity()));

        if (qualityCheckMap.isEmpty()) {
            return;
        }

        // 获取质检产品
        Set<Long> productIdSet = qualityCheckMap.keySet();

        // 验证质检数
        List<JdRefundNotQualityCheck> list = list(q -> q.in(JdRefundNotQualityCheck::getProductId, productIdSet));
        Map<Long, JdRefundNotQualityCheck> notQualityCheckMap = list.stream().collect(Collectors.toMap(JdRefundNotQualityCheck::getProductId, Function.identity()));

        String code = CodeEnum.REFUND_QUALITY_CHECK.getCode();

        Collection<JdRefundQualityCheck> jdRefundQualityChecks = qualityCheckMap.values();
        for (JdRefundQualityCheck jdRefundQualityCheck : jdRefundQualityChecks) {

            // 产品id
            Long productId = jdRefundQualityCheck.getProductId();
            Integer qualifiedCount = jdRefundQualityCheck.getQualifiedCount();
            Integer defectiveCount = jdRefundQualityCheck.getDefectiveCount();
            Integer scrappedCount = jdRefundQualityCheck.getScrappedCount();

            jdRefundQualityCheck.setCode(code);
            jdRefundQualityCheck.setScrappedToQualifiedCount(0);
            jdRefundQualityCheck.setScrappedToDefectiveCount(0);
            jdRefundQualityCheck.setResultScrappedCount(0);
            jdRefundQualityCheck.setScrappedHandleStatus(scrappedCount == 0 ? 0 : 1);

            // 待质检数量
            JdRefundNotQualityCheck jdRefundNotQualityCheck = notQualityCheckMap.get(productId);

            int notCheckCount = jdRefundNotQualityCheck == null ? 0 : jdRefundNotQualityCheck.getQuantity();
            int totalCheckCount = qualifiedCount + defectiveCount + scrappedCount;
            if (notCheckCount < totalCheckCount) {
                ProductInfo productInfo = productInfoService.getById(productId);

                throw new ServiceException("产品【" + productInfo.getCustomCode() + "】待质检数量为"
                        + notCheckCount + ", 当前质检数量为" + totalCheckCount);
            }

            // 修改待质检数量
            jdRefundNotQualityCheck.setQuantity(notCheckCount - totalCheckCount);

            // 修改库存
            if (qualifiedCount + defectiveCount > 0) {
                stockService.update(q -> q
                        .eq(Stock::getProductId, jdRefundQualityCheck.getProductId())
                        .eq(Stock::getWarehouseId, WAREHOUSE_ID)
                        // 良品入库
                        .setSql(qualifiedCount > 0, "quantity = quantity + " + qualifiedCount)
                        // 次品入库
                        .setSql(defectiveCount > 0, getUpdateStockSql(3, defectiveCount))
                        // 减少锁定库存
                        .setSql(qualifiedCount + defectiveCount > 0, getUpdateStockSql(2, qualifiedCount + defectiveCount))
                );
            }

        }

        // 保存质检记录
        qualityCheckService.saveBatch(jdRefundQualityChecks);

        // 修改待质检数量
        updateBatchById(list);
    }

    @Override
    public List<JdRefundNotQualityCheckVo> excelList(MultipartFile file) {
        // 备件条码列表
        Map<String, JdRefundNotQualityCheckVo> JdRefundNotQualityCheckMap = new HashMap<>();

        EasyExcel.read(ExcelUtil.getInputStream(file)).sheet(0).registerReadListener(new AnalysisEventListener<Map<Integer, String>>() {

            @Override
            public void invoke(Map<Integer, String> map, AnalysisContext analysisContext) {
                String productCustomCode = map.get(0);
                Assert.notEmpty(productCustomCode, "存在物品编码为空的数据");

                JdRefundNotQualityCheckVo vo = JdRefundNotQualityCheckMap.get(productCustomCode);
                Assert.empty(vo, "物品编码重复：" + productCustomCode);

                vo = new JdRefundNotQualityCheckVo();
                vo.setProductCustomCode(productCustomCode);
                vo.setQualifiedCount(Convert.toInt(map.get(1), 0));
                vo.setDefectiveCount(Convert.toInt(map.get(2), 0));
                vo.setScrappedCount(Convert.toInt(map.get(3), 0));

                JdRefundNotQualityCheckMap.put(productCustomCode, vo);
            }

            @Override
            public void doAfterAllAnalysed(AnalysisContext context) {
                //数据读取完毕
            }
        }).headRowNumber(1).doRead();

        if (JdRefundNotQualityCheckMap.isEmpty()) {
            return Collections.emptyList();
        }

        ArrayList<JdRefundNotQualityCheckVo> list = new ArrayList<>(JdRefundNotQualityCheckMap.values());

        Set<String> productCustomCodeSet = list.stream().map(JdRefundNotQualityCheckVo::getProductCustomCode).collect(Collectors.toSet());
        List<ProductInfo> productInfoList = productInfoService.list(q -> q.in(ProductInfo::getCustomCode, productCustomCodeSet));
        Assert.notEmpty(productInfoList, "物品编码不存在");

        Map<String, ProductInfo> productInfoMap = productInfoList.stream().collect(Collectors.toMap(ProductInfo::getCustomCode, Function.identity()));

        Set<Long> productIdSet = productInfoList.stream().map(BaseIdPo::getId).collect(Collectors.toSet());
        Map<Long, Integer> productQuantityMap = list(q -> q.in(JdRefundNotQualityCheck::getProductId, productIdSet))
                .stream().collect(Collectors.toMap(JdRefundNotQualityCheck::getProductId, JdRefundNotQualityCheck::getQuantity));

        for (JdRefundNotQualityCheckVo item : list) {
            ProductInfo productInfo = productInfoMap.get(item.getProductCustomCode());
            Assert.notEmpty(productInfo, "物品编码不存在：" + item.getProductCustomCode());

            item.setProductId(productInfo.getId());
            item.setProductName(productInfo.getName());
            item.setProductCustomCode(productInfo.getCustomCode());
            item.setProductSpec(productInfo.getSpec());
            item.setProductUnit(productInfo.getUnit());
            item.setQuantity(productQuantityMap.getOrDefault(productInfo.getId(), 0));
        }

        return list;
    }

}
