package com.fjhx.victoriatourist.service.jd.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.utils.Assert;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.item.util.excel.util.ExcelUtil;
import com.fjhx.victoriatourist.entity.jd.dto.JdRefundSelectDto;
import com.fjhx.victoriatourist.entity.jd.po.JdRefund;
import com.fjhx.victoriatourist.entity.jd.po.JdRefundDetail;
import com.fjhx.victoriatourist.entity.jd.po.JdRefundNotQualityCheck;
import com.fjhx.victoriatourist.entity.jd.vo.JdRefundVo;
import com.fjhx.victoriatourist.mapper.jd.JdRefundMapper;
import com.fjhx.victoriatourist.service.jd.JdRefundDetailService;
import com.fjhx.victoriatourist.service.jd.JdRefundNotQualityCheckService;
import com.fjhx.victoriatourist.service.jd.JdRefundService;
import com.fjhx.wms.entity.stock.po.Stock;
import com.fjhx.wms.service.stock.StockService;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 京东售后退货 服务实现类
 * </p>
 *
 * @author
 * @since 2024-09-25
 */
@Service
public class JdRefundServiceImpl extends ServiceImpl<JdRefundMapper, JdRefund> implements JdRefundService {

    // 仓库id
    public final static Long WAREHOUSE_ID = 1774616929466654722L;
    private static final String sql = "victoriatourist_json=JSON_SET(" +
            "victoriatourist_json,'$.{}',(json_extract(victoriatourist_json,'$.{}'){}{}))";
    private static final Map<Integer, String[]> typeMappings = new HashMap<Integer, String[]>() {{
        put(1, new String[]{"frozenQuantity", "+"});
        put(2, new String[]{"frozenQuantity", "-"});
        put(3, new String[]{"defectiveQuantity", "+"});
        put(4, new String[]{"defectiveQuantity", "-"});
    }};
    @Autowired
    private JdRefundDetailService jdRefundDetailService;
    @Autowired
    private ProductInfoService productInfoService;
    @Autowired
    private StockService stockService;
    @Autowired
    private JdRefundNotQualityCheckService jdRefundNotQualityCheckService;

    /**
     * @param type 1冻结新增 2冻结减少 3次品新增 4次品减少
     * @return sql
     */
    public static String getUpdateStockSql(Integer type, Number quantity) {
        String[] mapping = typeMappings.get(type);
        if (mapping == null) {
            throw new IllegalArgumentException("type参数错误");
        }
        return StrUtil.format(sql, mapping[0], mapping[0], mapping[1], quantity);
    }

    @Override
    public Page<JdRefundVo> getPage(JdRefundSelectDto dto) {

        Date beginTime = dto.getBeginTime();
        Date endTime = dto.getEndTime();
        String keyword = dto.getKeyword();

        IWrapper<JdRefund> wrapper = getWrapper()
                .eq("jrd", JdRefundDetail::getStatus, dto.getStatus())
                .between("jr", JdRefund::getApplyDateTime, beginTime, endTime)
                .keyword(keyword,
                        new SqlField("jr", JdRefund::getOutNo),
                        new SqlField("jr", JdRefund::getStoreName),
                        new SqlField("jrd", JdRefundDetail::getPartCode),
                        new SqlField("jrd", JdRefundDetail::getShipCode),
                        new SqlField("jrd", JdRefundDetail::getWareId),
                        new SqlField("jrd", JdRefundDetail::getWareName),
                        new SqlField("jrd", JdRefundDetail::getConfirmReceiptPeople)
                );

        return baseMapper.getPage(dto.getPage(), wrapper);
    }

    @DSTransactional
    @Override
    public void confirm(Long id) {
        JdRefundDetail jdRefundDetail = jdRefundDetailService.getById(id);
        Assert.notEmpty(jdRefundDetail, "京东售后明细不存在");

        Integer status = jdRefundDetail.getStatus();
        Assert.eqZero(status, "京东售后单已确认，备件条码：" + jdRefundDetail.getPartCode());

        // 获取产品编号
        String wareId = jdRefundDetail.getWareId();
        ProductInfo productInfo = productInfoService.getOne(q -> q.eq(ProductInfo::getCustomCode, wareId));
        Assert.notEmpty(productInfo, "未知产品编码：" + wareId + ",备件条码：" + jdRefundDetail.getPartCode());

        // key： product_id  value：产品数量
        Map<Long, Integer> map = new HashMap<>();

        // 添加质检信息
        setJdRefundQualityCheck(map, productInfo, jdRefundDetail, 1);

        // 修改为已确认
        jdRefundDetail.setStatus(StatusConstant.YES);
        jdRefundDetailService.updateById(jdRefundDetail);

        // 新增锁定库存
        Set<Long> productIdSet = map.keySet();

        Map<Long, JdRefundNotQualityCheck> notQualityCheckMap = jdRefundNotQualityCheckService
                .list(q -> q.in(JdRefundNotQualityCheck::getProductId, productIdSet))
                .stream().collect(Collectors.toMap(JdRefundNotQualityCheck::getProductId, Function.identity()));

        Map<Long, Stock> stockMap = stockService
                .list(q -> q.in(Stock::getProductId, productIdSet).eq(Stock::getWarehouseId, WAREHOUSE_ID))
                .stream().collect(Collectors.toMap(Stock::getProductId, Function.identity()));

        map.forEach((k, v) -> {

            // 添加未质检库存
            JdRefundNotQualityCheck jdRefundNotQualityCheck = notQualityCheckMap.get(k);
            if (jdRefundNotQualityCheck == null) {
                JdRefundNotQualityCheck saveJdRefundNotQualityCheck = new JdRefundNotQualityCheck();
                saveJdRefundNotQualityCheck.setProductId(k);
                saveJdRefundNotQualityCheck.setQuantity(v);
                jdRefundNotQualityCheckService.save(saveJdRefundNotQualityCheck);
            } else {
                jdRefundNotQualityCheckService.update(q -> q
                        .eq(BaseIdPo::getId, jdRefundNotQualityCheck.getId())
                        .setSql("quantity = quantity + " + v)
                );
            }

            // 修改库存
            Stock stock = stockMap.get(k);
            if (stock == null) {
                JSONObject victoriatouristJson = new JSONObject();
                victoriatouristJson.put("frozenQuantity", v);
                victoriatouristJson.put("defectiveQuantity", 0);

                Stock saveStock = new Stock();
                saveStock.setWarehouseId(WAREHOUSE_ID);
                saveStock.setProductId(k);
                saveStock.setQuantity(BigDecimal.ZERO);
                saveStock.setVictoriatouristJson(victoriatouristJson.toJSONString());
                stockService.save(saveStock);
            } else {
                stockService.update(q -> q
                        .eq(BaseIdPo::getId, stock.getId())
                        .setSql(getUpdateStockSql(1, v))
                );
            }

        });

    }

    @DSTransactional
    @Override
    public void cancelConfirm(Long id) {
        JdRefundDetail jdRefundDetail = jdRefundDetailService.getById(id);
        Assert.notEmpty(jdRefundDetail, "京东售后明细不存在");

        Integer status = jdRefundDetail.getStatus();
        Assert.eqOne(status, "京东售后单未确认");

        // 获取产品编号
        String wareId = jdRefundDetail.getWareId();
        ProductInfo productInfo = productInfoService.getOne(q -> q.eq(ProductInfo::getCustomCode, wareId));
        Assert.notEmpty(productInfo, "未知产品编码：" + wareId);

        // key： product_id  value：产品数量
        Map<Long, Integer> map = new HashMap<>();

        // 添加质检信息
        setJdRefundQualityCheck(map, productInfo, jdRefundDetail, 1);

        // 修改为待确认
        jdRefundDetail.setStatus(StatusConstant.NO);
        jdRefundDetailService.updateById(jdRefundDetail);

        // 新增锁定库存
        Set<Long> productIdSet = map.keySet();

        Map<Long, JdRefundNotQualityCheck> notQualityCheckMap = jdRefundNotQualityCheckService
                .list(q -> q.in(JdRefundNotQualityCheck::getProductId, productIdSet))
                .stream().collect(Collectors.toMap(JdRefundNotQualityCheck::getProductId, Function.identity()));

        Map<Long, Stock> stockMap = stockService
                .list(q -> q.in(Stock::getProductId, productIdSet).eq(Stock::getWarehouseId, WAREHOUSE_ID))
                .stream().collect(Collectors.toMap(Stock::getProductId, Function.identity()));

        map.forEach((k, v) -> {

            // 修改未质检库存
            JdRefundNotQualityCheck jdRefundNotQualityCheck = notQualityCheckMap.get(k);
            Assert.notEmpty(jdRefundNotQualityCheck, "未质检库存少于撤回售后退货数量，撤回失败");
            boolean update = jdRefundNotQualityCheckService.update(q -> q
                    .eq(BaseIdPo::getId, jdRefundNotQualityCheck.getId())
                    .setSql("quantity = quantity - " + v)
                    .ge(JdRefundNotQualityCheck::getQuantity, v)
            );
            Assert.eqTrue(update, "未质检库存少于撤回售后退货数量，撤回失败");

            // 修改库存
            Stock stock = stockMap.get(k);
            Assert.notEmpty(stock, "冻结少于撤回售后退货数量，撤回失败");
            boolean stockUpdate = stockService.update(Wrappers.<Stock>update()
                    .eq("id", stock.getId())
                    .setSql(getUpdateStockSql(2, v))
                    .geSql("json_extract(victoriatourist_json,'$.frozenQuantity')", v.toString())
            );
            Assert.eqTrue(stockUpdate, "冻结库存少于撤回售后退货数量，撤回失败");

        });

    }

    @DSTransactional
    @Override
    public void confirmExcelImport(MultipartFile file) {

        // 备件条码列表
        List<String> partCodeList = new ArrayList<>();

        EasyExcel.read(ExcelUtil.getInputStream(file)).sheet(0).registerReadListener(new AnalysisEventListener<Map<Integer, String>>() {

            @Override
            public void invoke(Map<Integer, String> map, AnalysisContext analysisContext) {
                String value = map.get(0);
                if (StrUtil.isBlank(value)) {
                    return;
                }
                partCodeList.add(value.toUpperCase().trim());
            }

            @Override
            public void doAfterAllAnalysed(AnalysisContext context) {
                //数据读取完毕
            }

        }).headRowNumber(1).doRead();

        if (partCodeList.isEmpty()) {
            return;
        }

        Map<String, Long> map = jdRefundDetailService.list(q -> q.in(JdRefundDetail::getPartCode, partCodeList))
                .stream().collect(Collectors.toMap(item -> item.getPartCode().toUpperCase().trim(), JdRefundDetail::getId));

        for (String partCode : partCodeList) {
            Long id = map.get(partCode);
            Assert.notEmpty(id, "备件条码不存在：" + partCode);
            confirm(id);
        }

    }

    /**
     * 赋值需要质检的商品
     */
    private void setJdRefundQualityCheck(Map<Long, Integer> map, ProductInfo productInfo, JdRefundDetail jdRefundDetail, Integer number) {
        String victoriatouristJson = productInfo.getVictoriatouristJson();
        JSONObject victoriatourist = JSONObject.parseObject(victoriatouristJson);

        // 如果是组合商品，拆分组合
        if (StatusConstant.YES.equals(victoriatourist.getInteger("combination"))) {
            List<JSONObject> list = victoriatourist.getJSONArray("productCombinationList").toJavaList(JSONObject.class);
            list.forEach(item -> {
                Long linkProductId = item.getLong("linkProductId");
                ProductInfo linkProductInfo = productInfoService.getById(linkProductId);
                Assert.notEmpty(linkProductInfo, "未知组合商品id：" + linkProductId);
                setJdRefundQualityCheck(map, linkProductInfo, jdRefundDetail, item.getInteger("linkQuantity") * number);
            });
        }
        // 不是组合
        else {
            Long productInfoId = productInfo.getId();
            map.put(productInfoId, map.getOrDefault(productInfoId, 0) + number);
        }
    }

}
