package com.fjhx.victoriatourist.service.jd.initializers;

import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.victoriatourist.entity.jd.vo.JdInfoVo;
import com.fjhx.victoriatourist.service.jd.JdApiService;
import com.fjhx.victoriatourist.service.jd.constants.RunParamConstant;
import com.ruoyi.common.utils.SecurityUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 京东初始化类
 *
 * @author hj
 * @date 2023/12/21 9:11
 */
@Slf4j
@Profile("prod")
@Component
@EnableScheduling
public class JdInitializer {

    @Resource
    private JdApiService jdApiService;

    @Scheduled(fixedDelay = RunParamConstant.SLEEP_ONE_MINUTE)
    public void jdTask() {
        SecurityUtils.setTenantId(RunParamConstant.tenantId);
        DynamicDataSourceContextHolder.push(SourceConstant.VICTORIATOURIST);

        JdInfoVo jdInfoVo = jdApiService.validJdInfo();

        try {
            jdApiService.startJdOrder(jdInfoVo);
            log.info("京东订单采集完成");
        } catch (Exception e) {
            log.error("京东订单采集出错：{}", e.getMessage(), e);
        }

        try {
            jdApiService.startJdBackOrder(jdInfoVo);
            log.info("京东退货单采集完成");
        } catch (Exception e) {
            log.error("京东退货单采集出错：{}", e.getMessage(), e);
        }

        try {
            jdApiService.refreshJdState(jdInfoVo);
            log.info("刷新京东状态完成");
        } catch (Exception e) {
            log.error("刷新京东状态出错：{}", e.getMessage(), e);
        }

        try {
            // https://jos.jd.com/apilistnewdetail?apiGroupId=86&apiId=19455&apiName=null
            jdApiService.refundInfo(jdInfoVo);
            log.info("京东收集售后退货列表完成");
        } catch (Exception e) {
            log.error("京东收集售后退货列表出错：{}", e.getMessage(), e);
        }

        try {
            jdApiService.initTenantDict();
            log.info("字典初始化完成");
        } catch (Exception e) {
            log.error("字典初始化出错：{}", e.getMessage(), e);
        }

        try {
            jdApiService.compensatingProductNon();
            log.info("补偿商品id完成");
        } catch (Exception e) {
            log.error("补偿商品id出错：{}", e.getMessage(), e);
        }

        SecurityUtils.clearTenantId();
        DynamicDataSourceContextHolder.poll();
    }

}
